#!/usr/bin/env bash


script_name="$(basename $(realpath $0))"
working_dir="$(dirname $(realpath $0))"

conf_file="${working_dir}/conf/${script_name%%.*}.conf"

LOCK_DIR="/tmp/$(basename $0).$$.LOCK"
log_file="${LOCK_DIR}/${script_name%%.*}.log"

backup_date="$(date +%Y-%m-%d-%H%M)"
declare -a files_backuped_arr=()
declare -a dirs_backuped_arr=()

# ----------
# Base Function(s)
# ----------

clean_up() {

   # Perform program exit housekeeping
   rm -rf "$LOCK_DIR"
   blank_line
   exit $1
}


echononl(){
   if $terminal ; then
      echo X\\c > /tmp/shprompt$$
      if [ `wc -c /tmp/shprompt$$ | awk '{print $1}'` -eq 1 ]; then
         echo -e -n "   $*\\c" 1>&2
      else
         echo -e -n "   $*" 1>&2
      fi
      rm /tmp/shprompt$$
   fi
}

fatal(){
   echo ""
   if $terminal ; then
      echo -e "   [ \033[31m\033[1mFatal\033[m ] $*"
   else
      echo -e "   [ Fatal ] $*"
   fi
   echo ""
   if $terminal ; then
      echo -e "             \033[1mScript terminated\033[m.."
   else
      echo -e "             Script terminated.."
   fi
   echo ""
   rm -rf $LOCK_DIR
   exit 1
}

error (){
   echo ""
   if $terminal ; then
      echo -e "   [ \033[31m\033[1mError\033[m ] $*"
   else
      echo "   [ Error ] $*"
   fi
   echo ""
}

warn (){
   if $LOGGING || $terminal ; then
      echo ""
      if $terminal ; then
         echo -e "   [ \033[33m\033[1mWarn\033[m ]  $*"
      else
         echo "   [ Warn ]  $*"
      fi
      echo ""
   fi
}

info (){
   if $LOGGING || $terminal ; then
      echo ""
      if $terminal ; then
         echo -e "   [ \033[32m\033[1mInfo\033[m ]  $*"
      else
         echo "   [ Info ]  $*"
      fi
      echo ""
   fi
}

ok (){
   if $LOGGING || $terminal ; then
      echo ""
      if $terminal ; then
         echo -e "   [ \033[32m\033[1mOk\033[m ]    $*"
      else
         echo "   [ Ok ]    $*"
      fi
      echo ""
   fi
}

echo_done() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mdone\033[m ]"
   fi
}
echo_ok() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mok\033[m ]"
   fi
}
echo_failed(){
   if $terminal ; then
      echo -e "\033[75G[ \033[1;31mfailed\033[m ]"
   fi
}
echo_skipped() {
   if $terminal ; then
      echo -e "\033[75G[ \033[33m\033[1mskipped\033[m ]"
   fi
}
echo_wait(){
   if $terminal ; then
      echo -en "\033[75G[ \033[5m\033[1m...\033[m ]"
   fi
}

trim() {
    local var="$*"
    var="${var#"${var%%[![:space:]]*}"}"   # remove leading whitespace characters
    var="${var%"${var##*[![:space:]]}"}"   # remove trailing whitespace characters
    echo -n "$var"
}

blank_line() {
   if $terminal ; then
      echo ""
   fi
}



# ----------
# - Jobhandling 
# ----------

# - Run 'clean_up' for signals SIGHUP SIGINT SIGTERM
# -
trap clean_up SIGHUP SIGINT SIGTERM

# - Create lock directory '$LOCK_DIR"
#
mkdir "$LOCK_DIR"


# ----------
# - Some checks ..
# ----------

# - Running in a terminal?
# -
if [[ -t 1 ]] ; then
   terminal=true
else
   terminal=false
fi


# ==========
# - Begin Main Script
# ==========

# ----------
# - Headline
# ----------

if $terminal ; then
   echo ""
   echo -e "\033[1m----------\033[m"
   echo -e "\033[32m\033[1mRunning script \033[m\033[1m$script_name\033[32m .. \033[m"
   echo -e "\033[1m----------\033[m"
fi


# ----------
#  Read Configurations from $conf_file
# ----------


# - Give your default values here
# -
LOGGING=false
BATCH_MODE=false
DEFAULT_REPOSITORY_NAME="ipt-gateway"
DEFAULT_REPOSITORY_BASE_PATH="/usr/local/src"
DEFAULT_IPV4_FIREWALL_SCRIPT="ipt-firewall-gateway"
DEFAULT_IPV6_FIREWALL_SCRIPT="ip6t-firewall-gateway"
DEFAULT_CONFIG_DIR="/etc/ipt-firewall"
DEFAULT_FIREWALL_CONFIGURATION_FILES="
   include_functions.conf
   load_modules_ipv4.conf
   load_modules_ipv6.conf
   logging_ipv4.conf
   logging_ipv6.conf
   post_decalrations.conf
   default_ports.conf
"

if [[ -f "$conf_file" ]]; then
   source "$conf_file"
else
   warn "No configuration file '$conf_file' present.\n
             Loading default values.."
fi

[[ -n "$REPOSITORY_NAME" ]] || REPOSITORY_NAME="$DEFAULT_REPOSITORY_NAME"
[[ -n "$REPOSITORY_BASE_PATH" ]] || REPOSITORY_BASE_PATH="${DEFAULT_REPOSITORY_BASE_PATH}"

[[ -n "$IPV4_FIREWALL_SCRIPT" ]] || IPV4_FIREWALL_SCRIPT="$DEFAULT_IPV4_FIREWALL_SCRIPT"
[[ -n "$IPV6_FIREWALL_SCRIPT" ]] || IPV6_FIREWALL_SCRIPT="$DEFAULT_IPV6_FIREWALL_SCRIPT"

[[ -n "$CONFIG_DIR" ]] || CONFIG_DIR="$DEFAULT_CONFIG_DIR"

[[ -n "$FIREWALL_CONFIGURATION_FILES" ]] || FIREWALL_CONFIGURATION_FILES="$DEFAULT_FIREWALL_CONFIGURATION_FILES"

REPOSITORY_PATH="${REPOSITORY_BASE_PATH}/$REPOSITORY_NAME"



# ----------
# - Main part of script
# ----------

if $terminal ; then
   echo ""
   echo ""
   echo -e "   \033[1mMain part of script ..\033[m"
   echo ""
fi

if [[ ! -d "$REPOSITORY_PATH" ]] ; then
	info "Repository '$REPOSITORY_NAME' is not present."
	clean_up 0
fi 

if $terminal ; then
   echo ""
   echo -e "   Update firewall scripts from local repository \033[1m$REPOSITORY_NAME\033[m  .."

   echo ""
fi

restart_ipv6_firewall=false
restart_ipv4_firewall=false

echononl "Copy firewall script '$IPV4_FIREWALL_SCRIPT' to /usr/local/sbin/ .."
if [[ -f "${REPOSITORY_PATH}/$IPV4_FIREWALL_SCRIPT" ]]; then
   if $(diff "${REPOSITORY_PATH}/$IPV4_FIREWALL_SCRIPT" "/usr/local/sbin/$IPV4_FIREWALL_SCRIPT" > /dev/null 2>&1) ; then
      echo_skipped
   else
      cp "${REPOSITORY_PATH}/$IPV4_FIREWALL_SCRIPT" /usr/local/sbin/ > "$log_file" 2>&1
      if [[ $? -gt 0 ]]; then
         echo_failed
         error "$(cat "$log_file")"
      else
         echo_ok
         restart_ipv4_firewall=true
      fi
   fi
else
   echo_skipped
   warn "No file '$IPV4_FIREWALL_SCRIPT' found at repository '$REPOSITORY_NAME'"
fi

echononl "Copy firewall script '$IPV6_FIREWALL_SCRIPT' to /usr/local/sbin/ .."
if [[ -f "${REPOSITORY_PATH}/$IPV6_FIREWALL_SCRIPT" ]]; then
   if $(diff "${REPOSITORY_PATH}/$IPV6_FIREWALL_SCRIPT" "/usr/local/sbin/$IPV6_FIREWALL_SCRIPT" > /dev/null 2>&1) ; then
      echo_skipped
   else
      cp "${REPOSITORY_PATH}/$IPV6_FIREWALL_SCRIPT" /usr/local/sbin/ > "$log_file" 2>&1
      if [[ $? -gt 0 ]]; then
         echo_failed
         error "$(cat "$log_file")"
      else
         echo_ok
         restart_ipv6_firewall=true
      fi
   fi
else
   echo_skipped
   warn "No file '$IPV6_FIREWALL_SCRIPT' found at repository '$REPOSITORY_NAME'"
fi


if $terminal ; then
   echo ""
   echo -e "   Update Configuration files from local repository \033[1m$REPOSITORY_NAME\033[m  .."

   echo ""
fi

for _file_name in ${FIREWALL_CONFIGURATION_FILES} ; do
   _file_src="${REPOSITORY_PATH}/conf/$_file_name"
   _file_dst="${CONFIG_DIR}/$_file_name"
   echononl "Copy firewall configuration file '$_file_name'.."
   if [[ -f "$_file_src" ]]; then
      if $(diff "$_file_src" "$_file_dst" > /dev/null 2>&1) ; then
         echo_skipped
      else
         cp "$_file_src" "$_file_dst" > "$log_file" 2>&1
         if [[ $? -gt 0 ]]; then
            echo_failed
            error "$(cat "$log_file")"
         else
            echo_ok
            restart_ipv6_firewall=true
            restart_ipv4_firewall=true
         fi
      fi
   else
      echo_skipped
      warn "No configuration file '$_file_name' found!"
   fi
done


blank_line


echononl "Restart IPv4 Firewall.."
if $restart_ipv4_firewall ; then
   /usr/local/sbin/$IPV4_FIREWALL_SCRIPT > /dev/null 2> "$log_file"
   if [[ $? -gt 0 ]]; then
      echo_failed
      error "$(cat "$log_file")"
   else
      echo_ok
   fi
else
   echo_skipped
fi

echononl "Restart IPv6 Firewall.."
if $restart_ipv6_firewall \
      && [[ -f "/etc/ipt-firewall/main_ipv6.conf" ]] \
      && [[ -f "/usr/local/sbin/$IPV6_FIREWALL_SCRIPT" ]]; then
   /usr/local/sbin/$IPV6_FIREWALL_SCRIPT > /dev/null 2> "$log_file"
   if [[ $? -gt 0 ]]; then
      echo_failed
      error "$(cat "$log_file")"
   else
      echo_ok
   fi
else
   echo_skipped
fi

info "Configuration files \033[1mmain_ipv[4|6].conf\033[m are not considered."


clean_up 0
